/*
 * channeltab.cpp
 *
 * Copyright 2001 Matthew Stephenson
 * 
 * This file is part of Zic.
 * 
 * Zic is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Zic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Zic; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Look, feel, and userlist icons shamelessly stolen from X-Chat 
 * (www.xchat.org)
 * 
 */

#include "channeltab.h"

channeltab::channeltab( zic *parent, ircchannel *channel, QString label = "<none>" )
  : irctab( parent, label )
{

  //channel = new ircchannel( label );

  this->channel = channel;

  hbox = new QHBox( vbox );

  outputbox = new ircview( hbox );
  outputbox->setVScrollBarMode(QScrollView::AlwaysOn);
  outputbox->show();

  userlistbutton = new QPushButton(">", outputbox);
  outputbox->setCornerWidget(userlistbutton);
  connect(userlistbutton, SIGNAL(clicked()), this, SLOT(showHideUserlist()));

  userlist = new ircuserlist( hbox );
  userlist->setMaximumWidth(USERLISTWIDTH);
  userlist->show();
  b_userlistvisible = true;

  hbox->show();
  
  inputbox = new QLineEdit( vbox );
  connect(inputbox, SIGNAL(returnPressed()), this, SLOT(userInput()));
  inputbox->show();
  inputbox->setFocus();

  emit amTopTab(this);

}

channeltab::~channeltab()
{
  //delete channel;
}

void channeltab::showHideUserlist()
{
  if(b_userlistvisible) {
    userlist->setMaximumWidth(0);
    userlistbutton->setText("<");
    b_userlistvisible = false;
  }
  else {
    userlist->setMaximumWidth(USERLISTWIDTH);
    userlistbutton->setText(">");
    b_userlistvisible = true;
  }
  // vile hack necessary to scroll the damn thing back to the bottom
  outputbox->resize(0,0);
  outputbox->redraw();
}

void channeltab::join( ircuser *user )
{
  outputbox->join( user->nick() );
  channel->join( user );
  userlist->update( channel );
}

void channeltab::part( ircuser *user, QString reason )
{
  outputbox->part( user->nick(), reason );
  channel->part( user );
  userlist->update( channel );
}

void channeltab::quit( ircuser *user, QString reason )
{
  outputbox->quit( user->nick(), reason );
  channel->part( user );
  userlist->update( channel );
  
}

void channeltab::kick( ircuser *user, QString nick, QString reason )
{
  outputbox->kick( user->nick(), nick, reason );
  ircuser *target = new ircuser(nick);
  channel->part( target );
  delete target;
  userlist->update( channel );

}

void channeltab::names( QStringList *names )
{
  channel->parseNames( names );
}

void channeltab::endofnames()
{
  channel->endOfNames();
  userlist->update( channel );
}

void channeltab::mode(ircmode * mode)
{
  if(mode->target().lower() == s_label.lower()) {
    channel->mode(mode);
    userlist->update( channel );
  }

}

void channeltab::nick(QString from, QString to)
{
  outputbox->nick(from, to);
  channel->nick(from, to);
  userlist->update( channel );
}
