/*
 * connectbox.cpp
 *
 * Copyright 2001 Matthew Stephenson
 * 
 * This file is part of Zic.
 * 
 * Zic is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Zic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Zic; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Look, feel, and userlist icons shamelessly stolen from X-Chat 
 * (www.xchat.org)
 * 
 */


#include "connectbox.h"

connectbox::connectbox(QString *nick, QString *server, uint *port, QStringList *channels)
  : QDialog(0,"foo",TRUE) 
{
  
  this->nick = nick;
  this->server = server;
  this->port = port;
  this->channels = channels;

  //setFixedSize(200,200);
  setCaption("Connect to IRC Server");

  // Set up the initial vbox and tabwidget
  
  vbox = new QVBox(this);
  vbox->setMinimumSize(200,150);

  tabwidget = new QTabWidget(vbox);

  connectButton = new QPushButton("Connect", vbox);
  connect(connectButton, SIGNAL(clicked()), this, SLOT(ConnectButtonPush()));

  // Set up the Presets page

  presetHbox = new QHBox(tabwidget);
  presetListBox = new QListBox(presetHbox);
  presetVbox = new QVBox(presetHbox);
  
  presetAddButton = new QPushButton("Add", presetVbox);
  connect(presetAddButton, SIGNAL(clicked()), this, SLOT(addPreset()));
  presetEditButton = new QPushButton("Edit", presetVbox);
  connect(presetEditButton, SIGNAL(clicked()), this, SLOT(editPreset()));
  presetDelButton = new QPushButton("Del", presetVbox);
  connect(presetDelButton, SIGNAL(clicked()), this, SLOT(delPreset()));

  presets = new QList<preset>;
  presets->setAutoDelete(true);

  tabwidget->addTab( presetHbox, "Presets");
  
  // Set up the Quick connect page
  
  quickVbox = new QVBox(tabwidget);
  
  quickGrid = new QGrid(2,quickVbox);
  quickGrid->setSpacing(5);

  new QLabel("Nick:", quickGrid);
  quickNickText = new QLineEdit(quickGrid);
  connect(quickNickText, SIGNAL(textChanged(const QString &)), 
	  this, SLOT(setNick(const QString &)));

  new QLabel("Server:",quickGrid);
  quickServerText = new QLineEdit(quickGrid);
   connect(quickServerText, SIGNAL(textChanged(const QString &)), 
	  this, SLOT(setServer(const QString &)));
  
  tabwidget->addTab( quickVbox, "Quick");

  // Now populate stuff

  loadSettings(); 
  populateListbox();
  quickNickText->setText(*nick);
  quickServerText->setText(*server);

 
}

connectbox::~connectbox()
{

}

void connectbox::setNick(const QString & nick)
{
  *(this->nick) = nick;
}

void connectbox::setServer(const QString & server)
{
  *(this->server) = server;
}

void connectbox::ConnectButtonPush()
{
  accept();
}

void connectbox::loadSettings()
{
  QString temp;
  preset *pst;

  Config settings("zic", Config::User);
  settings.setGroup("QuickConnect");
  *nick = settings.readEntry("Nick", "ZicUser");
  *server = settings.readEntry("Server", "irc.irctoo.net");

  
  presets->clear();
  settings.setGroup("Presets");
  Config cfgPresets("zic", Config::User); 
  for(int i=0; settings.hasKey(QString::number(i)); i++) {
    cfgPresets.setGroup("Preset" + QString::number(i));
    pst = new preset();
    pst->name = settings.readEntry(QString::number(i));
    pst->nick = cfgPresets.readEntry("nick", "ZicUser");
    pst->server = cfgPresets.readEntry("server", "irc.irctoo.net");
    pst->port = cfgPresets.readEntry("port", "6667").toUInt();
    pst->password = cfgPresets.readEntry("password", ""); //FIXME crypt this?
    pst->channels = QStringList::split
      (QRegExp("[, ]"), cfgPresets.readEntry("channels",""));
    presets->append(pst);
  }

}


bool connectbox::ConnectQuick()
{
  if(this->nick->length() == 0) {
    QMessageBox::information(0, "Zic", "You must specify a nickname");
    return false;
  }
  if(this->nick->contains(' ')) {
    QMessageBox::information(0, "Zic", "You can't have a space in your nick");
    return false;
  }

  if(this->server->length() == 0) {
    QMessageBox::information(0, "Zic", "You must specify an IRC server");
    return false;
  }
  if(this->server->contains(' ')) {
    QMessageBox::information(0, "Zic", "You can't have a space\nin the server name");
    return false;
  }

  // Save the settings to a Config
  Config settings("zic", Config::User);
  settings.setGroup("QuickConnect");
  settings.writeEntry("Nick", *(this->nick));
  settings.writeEntry("Server", *(this->server));

  *port = DEFAULT_PORT;

  return true;
}

bool connectbox::ConnectPreset()
{
  //FIXME this assumes that the preset args are sane. Do some sanity-checking
  int i;

  if((i = presetListBox->currentItem()) == -1)
    return false;
  
  *nick = presets->at(i)->nick;
  *server = presets->at(i)->server;
  *port = presets->at(i)->port;
  *channels = presets->at(i)->channels;
  return true;
}

void connectbox::accept()
{
  if(tabwidget->currentPage() == quickVbox) {
    if(ConnectQuick()) {
     done(QDialog::Accepted);
    }
  }
  else if(tabwidget->currentPage() == presetHbox) {
    if(ConnectPreset()) {
      done(QDialog::Accepted);
    }
  }
}


void connectbox::addPreset()
{
  presetbox *pb = new presetbox(presets, presetListBox->currentItem(), true);
  pb->exec();
  if(pb->result() == QDialog::Rejected) {  
    delete pb;
    qWarning("Canceled"); //FIXME remove
    return;
  }
  
  savePresets();

  populateListbox();
  delete pb; 
}


void connectbox::editPreset()
{
  if(presetListBox->currentItem() == -1)
    return;

  presetbox *pb = new presetbox(presets, presetListBox->currentItem(), false);
  pb->exec();
  if(pb->result() == QDialog::Rejected) {  
    delete pb;
    qWarning("Canceled"); //FIXME remove
    return;
  }

  savePresets();

  populateListbox();
  delete pb; 
}


void connectbox::populateListbox()
{
  presetListBox->clear();
  for(uint i=0; i < presets->count(); i++) {
    presetListBox->insertItem(presets->at(i)->name);
  }
}


void connectbox::savePresets()
{
  Config *cfg;

  cfg = new Config("zic", Config::User); 
  cfg->setGroup("Presets");
  cfg->clearGroup();
  delete cfg;
  

  for(uint i=0; i < presets->count(); i++) {
    cfg = new Config("zic", Config::User); 
    cfg->setGroup("Presets");
    cfg->writeEntry(QString::number(i), presets->at(i)->name);
    delete cfg;

    cfg = new Config("zic", Config::User); 
    cfg->setGroup("Preset" + QString::number(i));
    cfg->writeEntry("nick", presets->at(i)->nick);
    cfg->writeEntry("server", presets->at(i)->server);
    cfg->writeEntry("port", QString::number(presets->at(i)->port));
    cfg->writeEntry("password", presets->at(i)->password);
    cfg->writeEntry("channels", presets->at(i)->channels.join(","));
    delete cfg;
  }

}


void connectbox::delPreset()
{
  if(presetListBox->currentItem() == -1)
    return;

  // FIXME do confirmation messagebox here

  presets->remove(presetListBox->currentItem());

  savePresets();

  populateListbox();
}
