/*
 * connection.h
 *
 * Copyright 2001 Matthew Stephenson
 * 
 * This file is part of Zic.
 * 
 * Zic is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Zic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Zic; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Look, feel, and userlist icons shamelessly stolen from X-Chat 
 * (www.xchat.org)
 * 
 */

#ifndef CONNECTION_H
#define CONNECTION_H

#include <qwidget.h>
#include <qtabbar.h>
#include <qstring.h>
#include <qtextstream.h>
#include <qstringlist.h>
#include "socket.h"
#include "options.h"
#include "ircmessage.h"
#include "ircuser.h"

class connection : public QObject
{
  Q_OBJECT
public:
  connection();
  void connectToServer( QString server, int port, QString nick, 
		QString username, QString realname);
  void parseCommand( QString command , QString fromChannel = "");
  QString getServerNick();
  bool connected();
  bool loggedIn();
  bool isChannel(const QString name);
  void sendPrivMsg( QString to, QString message );
  void sendCtcp( QString to, QString message );
  void sendCtcpNotice( QString to, QString message );

private:
  void parseLine( QString line );
  void emitNames( ircmessage *msg );
  void parseModes( ircmessage *msg );
  void parseNick( ircmessage *msg );

signals:
  void gotRawLine( QString line );
  void disconnected();
  void debug( QString line );
  void privmsg( ircmessage *msg );
  void notice( ircmessage *msg );
  void message( ircmessage *msg );
  void join( ircmessage *msg );
  void part( ircmessage *msg );
  void quit( ircmessage *msg );
  void kick( ircmessage *msg );
  void rpl_namreply( QString channel, QStringList *names );
  void rpl_endofnames( QString channel );
  void modes( ircmode * );
  void mode( ircmode * );
  void nick( ircmessage *msg );
  void errormsg( QString msg );
  void ctcp( ircmessage *msg );
  void ctcpnotice( ircmessage *msg );

private slots:
  void login();
  void readSocket(); 
  void socketDisconnect();

public:
  ircuser *me;
  QStringList autojoin;
  
private:
  socket *sock;  
  QString serverIP;
  QString serverPort;
  QString serverNick;
  bool b_connected;
  bool b_loggedIn;
};



#endif
