/*
 * ircchannel.cpp
 *
 * Copyright 2001 Matthew Stephenson
 * 
 * This file is part of Zic.
 * 
 * Zic is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Zic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Zic; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Look, feel, and userlist icons shamelessly stolen from X-Chat 
 * (www.xchat.org)
 * 
 */

#include "ircchannel.h"

ircchannel::ircchannel( QString name )
{
  s_name = name;
  l_users.setAutoDelete(true);
  
}


ircchannel::~ircchannel()
{
  
}

QString ircchannel::name() 
{
  return s_name;
}

void ircchannel::parseNames( QStringList *names )
{
  
  if(!(b_gettingnames)) {
    b_gettingnames = true;
    l_users.clear();
  }
  
  QString flags;
  for ( QStringList::Iterator it = names->begin(); it != names->end(); ++it ) {
    flags = "";
    while(((*it).left(1) == "@") 
	  || ((*it).left(1) == "%") 
	  || ((*it).left(1) == "+")) {
      flags.append((*it).left(1));
      *it = (*it).right((*it).length() - 1);
    }
    l_users.append(new channeluser(*it, flags));
  }
  
}


void ircchannel::endOfNames()
{
  l_users.sort();
  b_gettingnames = false;
}

QList<channeluser> * ircchannel::users()
{
  return &l_users;
}

channeluser * ircchannel::findUser(const QString nick )
{
  channeluser *ref = new channeluser( nick );
  
  int i = -1;
  if((i = l_users.find( ref )) > -1)
    return l_users.at(i);
  else
    return 0;
}


void ircchannel::mode( ircmode * mode )
{
  if(channeluser *user = findUser(mode->arg(0))) {
    if(mode->modes() == "+o") {
      user->op();
    }
    else if(mode->modes() == "-o") {
      user->deop();
    }
    else if(mode->modes() == "+v") {
      user->voice();
    }
    else if(mode->modes() == "-v") {
      user->devoice();
    }
    else if(mode->modes() == "+h") {
      user->hop();
    }
    else if(mode->modes() == "-h") {
      user->dehop();
    }
    else {
       qWarning("unknown mode for user in this channel");
    }
  }
  l_users.sort();
}

void ircchannel::join( ircuser *user )
{
  //FIXME I should probably use a constructor that copies the values across
  //here
  QString nickuserhost = user->nick() + "!" + 
    user->username() + "@" +
    user->host();

  channeluser *cuser = new channeluser(nickuserhost);
  
  l_users.append(cuser);
  l_users.sort();

}

void ircchannel::part( ircuser *user )
{
  
  if(channeluser *cuser = findUser( user->nick())) {
    l_users.remove(cuser);
    l_users.sort(); // FIXME is this necessary?
  }

}

void ircchannel::nick( QString from, QString to )
{
  if(channeluser *cuser = findUser( from )) {
    cuser->set_nick(to);
  }
  l_users.sort();
}
