/*
 * ircmessage.cpp
 *
 * Copyright 2001 Matthew Stephenson
 * 
 * This file is part of Zic.
 * 
 * Zic is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Zic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Zic; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Look, feel, and userlist icons shamelessly stolen from X-Chat 
 * (www.xchat.org)
 * 
 */

#include "ircmessage.h"

ircmessage::ircmessage( QString line )
  : p_from(0)
{

  // strip any CRs
  while((line.right(1) == "\n") || (line.right(1) == "\r")) 
    line = line.left(line.length() - 1);

  QString temp;

  QTextIStream stream( &line );

  stream >> temp;
  
  // If it has a : on the front, place this field in the "from" field
  if(temp.left(1) == ":") {
    p_from = new ircuser( temp );
    stream >> temp;
  }
  else {
    p_from = new ircuser(""); // whoo, scary, a message from nobody
  }
  
  s_command = temp;

  while(!(stream.atEnd())) {
    stream >> temp;
    s_param << temp;
  }
  
  // Now check it's not a CTCP message
  b_is_ctcp = false;
  if((s_command.lower() == "privmsg") || (s_command.lower() == "notice"))  { 
    if((this->params(1, true).left(1) == QChar(1)) &&
       (this->params(1, true).right(1) == QChar(1))) {
      // Strip out the \001 characters
      s_param[1] = s_param[1].replace(QRegExp(QChar(1)), "");
      s_param[s_param.count() - 1] = s_param[s_param.count() - 1].replace
	(QRegExp(QChar(1)), "");
      b_is_ctcp = true;
    }
  }

}

ircmessage::~ircmessage()
{
  delete p_from;
}

ircuser *ircmessage::from()
{
  return p_from;
}

QString ircmessage::command()
{
  return s_command;
}


QString ircmessage::params( bool stripcolon )
{
  return params( 0, s_param.count(), stripcolon );
}


QString ircmessage::params( int start, int num, bool stripcolon )
{
  int i;
  QString result, temp;
  QTextOStream stream(&result);
  bool strippedcolon = false;

  for(i=start;i<(start + num);i++) {
    temp = s_param[i] + " ";
    if(stripcolon && !strippedcolon)
      strippedcolon = stripcolons(&temp);
    stream << temp;
  }

  result = result.left(result.length() - 1);

  return result;
}

QString ircmessage::params( int start, bool stripcolon )
{
  return params(start, s_param.count() - start, stripcolon);
}


QString ircmessage::param( int num, bool stripcolon )
{
  if(stripcolon) {
    QString temp = s_param[num];
    stripcolons(&temp);
    return temp;
  }
  else
    return s_param[num];
}

bool ircmessage::stripcolons( QString *text )
{
  if(text->left(1) == ":") {
    *text = text->right(text->length() - 1);
    return true;
  }
  else
    return false;
}

int ircmessage::paramcount()
{
  return s_param.count();
}

bool ircmessage::is_ctcp()
{
  return b_is_ctcp;
}







ircmode::ircmode(ircmessage *msg)
{
  u_from = new ircuser(msg->from()->nick());
  s_target = msg->param(0);
  s_modes = msg->param(1, true);
  if(msg->paramcount() > 2) {
    for(int i = 2; i < msg->paramcount(); i++) {
      s_arg << msg->param(i, true);
    }
  }
}

ircmode::~ircmode()
{
  delete u_from;
}


QString ircmode::target()
{
  return s_target;
}

ircuser * ircmode::from()
{
  return u_from;
}

QString ircmode::modes()
{
  return s_modes;
}

QString ircmode::arg( int num )
{
  //FIXME bounds checking
  return s_arg[num];
}


QString ircmode::args()
{
  QString result, temp;
  QTextOStream stream(&result);

  for(uint i=0; i<s_arg.count(); i++) {
    temp = s_arg[i] + " ";
    stream << temp;
  }

  result = result.left(result.length() - 1);
  
  return result;

}
