/*
 * ircsession.h
 *
 * Copyright 2001 Matthew Stephenson
 * 
 * This file is part of Zic.
 * 
 * Zic is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Zic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Zic; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Look, feel, and userlist icons shamelessly stolen from X-Chat 
 * (www.xchat.org)
 * 
 */

#ifndef IRCSESSION_H
#define IRCSESSION_H

#include <qlist.h>
#include <qtimer.h>
#include "zic.h"
#include "irctab.h"
#include "plaintab.h"
#include "channeltab.h"
#include "options.h"
#include "connection.h"
#include "ircmessage.h"
#include "ircchannel.h"

class ircsession : public QObject
{
  Q_OBJECT

public:
  ircsession( zic *parent );
  void showConnectBox();

private:
  plaintab * addNewPlainTab( QString label );
  channeltab * addNewChannelTab( QString label );
  void removeTab( irctab *tab );
  irctab * findTab( QString name );
  ircchannel * findChannel( QString name );

public slots:
  void userInput( QString text, irctab* from );
  void socketDisconnect(); 
  void errormsg( QString msg );
  void parsePrivmsg( ircmessage *msg );
  void parseNotice( ircmessage *msg );
  void parseMessage( ircmessage *msg );
  void parseAction( ircmessage *msg );
  void parseJoin( ircmessage *msg );
  void parsePart( ircmessage *msg );
  void parseQuit( ircmessage *msg );
  void parseKick( ircmessage *msg );
  void parseNick( ircmessage *msg );
  void parseCtcp( ircmessage *msg );
  void parseMode( ircmode *mode );
  void newTopTab( irctab *tab );
  void removeTab();
  void rpl_namreply( QString channel, QStringList *names );
  void rpl_endofnames( QString channel );

private:
  zic *parent;
  QList<irctab> tabs;
  plaintab *servertab;
  irctab *toptab;
  connection *conn;
  irctab *tabtoremove; // see nasty removeTab() hack
  QList<ircchannel> channels;

#ifdef IRCDEBUG
  plaintab *debugtab;
  plaintab *rawtab;
#endif

};



#endif
