/*
 * irctab.cpp
 *
 * Copyright 2001 Matthew Stephenson
 * 
 * This file is part of Zic.
 * 
 * Zic is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Zic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Zic; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Look, feel, and userlist icons shamelessly stolen from X-Chat 
 * (www.xchat.org)
 * 
 */

#include "irctab.h"

irctab::irctab( zic *parent , QString label = "<none>")
  : QWidget( parent->channelStack )
{
  
  this->parent = parent;
  this->s_label = label;

  tab = new QTab( label );
  tabID = parent->tabBar->addTab(tab);
  parent->tabBar->setCurrentTab(tab);
  
  vbox = new QVBox( parent->channelStack );
  parent->channelStack->addWidget( vbox, 0 ); //FIXME should not be 0

  parent->channelStack->raiseWidget( vbox );

  connect(parent->tabBar, SIGNAL(selected(int)), this, SLOT(tabBarClicked(int)));
  emit amTopTab( this );

}

irctab::~irctab()
{
  parent->channelStack->removeWidget(vbox);
  delete vbox;

  // I THINK that removeTab deletes tab. 
  // if not, not deleting tab is a memory leak here.
  parent->tabBar->removeTab(tab);

  // poke the tabbar into poking the widgetstack into showing the right window :)
  parent->tabBar->show();
}

QString irctab::label()
{
  return s_label;
}



void irctab::tabBarClicked( int id )
{
  if(id == tabID) {
    parent->channelStack->raiseWidget( vbox );
    inputbox->setFocus();
    outputbox->redraw();
    emit amTopTab( this );
  }
}

void irctab::userInput()
{
  emit gotUserInput(inputbox->text(), this);
  inputbox->clear();
}

void irctab::addRawLine( QString line )
{
  // strip of any CR's or LF's
  while((line.right(1) == "\n") || (line.right(1) == "\r"))
    line = line.left(line.length() - 1);
  
  // NB: seemingly append() adds a \n anyway. hence the comment. duh.
  // line += "\n"; 

  outputbox->addLine(line);
}


// Hmm, I'm thinking some of these ought to be signals connected
// to signals. Hey ho, I'll do it later...

void irctab::privmsg( QString from, QString msg )
{
  outputbox->privmsg( from, msg );
}

void irctab::action( QString from, QString msg )
{
  outputbox->action( from, msg );
}

 
void irctab::notice( QString from, QString msg )
{
  outputbox->notice( from, msg );
}

void irctab::message( QString msg )
{
  outputbox->message( msg );
}

void irctab::quit( ircuser *user, QString reason )
{
  outputbox->quit( user->nick(), reason );
}

void irctab::modes(ircmode * mode)
{
  if(mode->target().lower() == s_label.lower()) 
    outputbox->modes(mode);
}

void irctab::nick( QString from, QString to )
{
  outputbox->nick(from, to);
}
