/*
 * ircuser.cpp
 *
 * Copyright 2001 Matthew Stephenson
 * 
 * This file is part of Zic.
 * 
 * Zic is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Zic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Zic; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Look, feel, and userlist icons shamelessly stolen from X-Chat 
 * (www.xchat.org)
 * 
 */

#include "ircuser.h"

ircuser::ircuser()
{

}


ircuser::ircuser( QString nickuserhost )
{

  // strip off any : on the beginning
  if(nickuserhost.left(1) == ":")
    nickuserhost = nickuserhost.right(nickuserhost.length() - 1);

  int i,j;
  i = nickuserhost.find("!");

  if(i == -1) { // there's no !, so assume it's all one nick
    s_nick = nickuserhost;
    return;
  }

  j = nickuserhost.find("@"); 
  s_nick = nickuserhost.left(i);
  s_username = nickuserhost.mid( i+1, j-i-1 );
  s_host = nickuserhost.right( nickuserhost.length() - j - 1 );

}


void ircuser::set_nick(QString newnick)
{
  s_nick = newnick;
}

void ircuser::set_username(QString newusername)
{
  s_username = newusername;
}

void ircuser::set_hostname(QString newhostname)
{
  s_host = newhostname;
}

void ircuser::set_realname(QString newrealname)
{
  s_realname = newrealname;
}


QString ircuser::nick()
{
  return s_nick;
}

QString ircuser::username()
{
  return s_username;
}

QString ircuser::host()
{
  return s_host;
}

QString ircuser::realname()
{
  return s_realname;
}

bool ircuser::operator==( ircuser u )
{
  return (this->nick().lower() == u.nick().lower());
}

bool ircuser::operator<( ircuser u )
{
  return (this->nick().lower() < u.nick().lower());
}



channeluser::channeluser()
{
  b_is_op = false;
  b_is_hop = false;
  b_is_voice = false;
}
 
channeluser::channeluser( QString nickuserhost, QString flags = "")
  : ircuser( nickuserhost )
{
  if(flags.contains("@"))
    b_is_op = true; 
  else
    b_is_op = false;

  if(flags.contains("%"))
    b_is_hop = true;
  else
    b_is_hop = false;

  if(flags.contains("+"))
    b_is_voice = true;
  else
    b_is_voice = false;
}

bool channeluser::is_op()
{
  return b_is_op;
}

void channeluser::op()
{
  b_is_op = true;
}

void channeluser::deop()
{
  b_is_op = false;
}

bool channeluser::is_hop()
{
  return b_is_hop;
}

void channeluser::hop()
{
  b_is_hop = true;
}

void channeluser::dehop()
{
  b_is_hop = false;
}

bool channeluser::is_voice()
{
  return b_is_voice;
}

void channeluser::voice()
{
  b_is_voice = true;
}

void channeluser::devoice()
{
  b_is_voice = false;
}


bool channeluser::operator==( channeluser u )
{

  // NB: I have deliberately left out comparison of channel modes from here
  // I don't think it will make any difference to the sorting of the nicklist
  // since no two users should have the same nick anyway. Leaving it out allows
  // easy comparison of users, especially in ircchannel::findUser()
  return (this->nick().lower() == u.nick().lower());
}

bool channeluser::operator<( channeluser u )
{
  if(!(this->is_op()) && u.is_op())
     return false;
  if(this->is_op() && !(u.is_op()))
     return true;
  if(!(this->is_hop()) && u.is_hop())
     return false;
  if(this->is_hop() && !(u.is_hop()))
     return true;
  if(!(this->is_voice()) && u.is_voice())
     return false;
  if(this->is_voice() && !(u.is_voice()))
     return true;

  return (this->nick().lower() < u.nick().lower());
}
