/*
 * ircview.cpp
 *
 * Copyright 2001 Matthew Stephenson
 * 
 * This file is part of Zic.
 * 
 * Zic is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Zic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Zic; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Look, feel, and userlist icons shamelessly stolen from X-Chat 
 * (www.xchat.org)
 * 
 */

#include "ircview.h"

ircview::ircview( QWidget *parent )
  : QTextView::QTextView( parent )
{
  setHScrollBarMode(QScrollView::AlwaysOff);
}


void ircview::addLine(QString line)
{
  while((line.right(1) == "\n") || line.right(1) == "\r")
    line = line.left(line.length() - 1);
  
  //line += "<br>\n";

  append(line);
  redraw();
}


void ircview::redraw()
{

  // terrible hack necessary because the append() function of QTextView
  // just doesn't work. Doesn't repaint properly, and the scrollbar doesn't
  // follow the size of the text...
  append(" ");
  //repaint();
  ensureVisible(0,contentsHeight(),0,0); //FIXME need better scrolling logic
  repaintContents(contentsX(),contentsY(),contentsWidth(), contentsHeight());
}

void ircview::privmsg( QString from, QString msg )
{
  addLine("&lt;<font color=\"#0000ff\">" + from + "</font>&gt; " + render(msg));
}

void ircview::action( QString from, QString msg )
{
  addLine("<font color=\"#0000ff\">*</font> " + from + " " + render(msg));
}

void ircview::notice( QString from, QString msg )
{
  addLine("-<font color=\"#880088\">" + from + "</font>- " + render(msg));
}
 
void ircview::message( QString msg )
{
  addLine("<font color=\"#DD8888\">-</font> " + msg);
}

void ircview::join( QString nick )
{
  addLine("<font color=\"#7755DD\">-&gt;</font> " + nick + " has joined");
}

void ircview::part( QString nick, QString reason )
{
  if(reason.length() == 0) {
    addLine("<font color=\"#7755DD\">&lt;-</font> " + nick + " has left");
  }
  else {
    addLine("<font color=\"#7755DD\">&lt;-</font> " + nick + " has left (" + reason + ")");
  }
}

void ircview::quit( QString nick, QString reason )
{
  addLine("<font color=\"#7755DD\">&lt;-</font> " + nick + " has quit (" + reason + ")");
}

void ircview::kick( QString by, QString nick, QString reason )
{
  if(reason.length() == 0) {
    addLine("<font color=\"#7755DD\">&lt;-</font> " + by + " has kicked " + nick);
  }
  else {
    addLine("<font color=\"#7755DD\">&lt;-</font> " + by + " has kicked " + nick +
	    " (" + reason + ")" );
  }
}

void ircview::modes( ircmode * mode )
{
  addLine(mode->from()->nick() + " sets modes " + mode->target() + " " +
	  mode->modes() + " " + mode->args());
}

void ircview::nick( QString from, QString to )
{
  addLine(from + " is now known as " + to);
}

QString ircview::render( QString line )
{
  bool bold = false;
  bool underline = false;
  bool colour = false;
  QString output;
  QChar c;
  int colourcode=0;

  for(uint i=0; i<line.length(); i++) {
    c = line.at(i);
    //qWarning(QString::number(c.latin1()).latin1());
    switch (c.latin1()) {
    case '<':
      output.append("&lt;");
      break;
    case '>':
      output.append("&gt;");
      break;
    case 2: //bold text
      if(bold) {
	output.append("</b>");
	bold = false;
      }
      else {
	output.append("<b>");
	bold = true;
      }
      break;
    case 31: //underlined text
      if(underline) {
	output.append("</u>");
	underline = false;
      }
      else {
	output.append("<u>");
	underline = true;
      }
      break;
    case 15: // "normal" text
      if(underline) {
	output.append("</u>");
	underline = false;
      }
      if(bold){
	output.append("</b>");
	bold = false;
      }
      if(colour) {
	output.append("</font>");
	colour = false;
      }
      break;
    case 3: // Colour
      if(colour) 
	output.append("</font>");
      i++;
      if(line.at(i+1).digitValue() > -1) {
	colourcode = (10 * line.at(i).digitValue());
	i++;
      }
      colourcode = colourcode + line.at(i).digitValue();
      //FIXME: this just skips over the ,5 bit of things like %C4,5
      //These are background colours and should be parsed
      //(If QTextview will allow this, of course)
      if(line.at(i+1) == ',') {
	i++;
	if(line.at(i+1).digitValue() > -1)
	  i++;
	if(line.at(i+1).digitValue() > -1)
	  i++;
      }
      //qWarning("Got colour: " + QString::number(colourcode));
      switch(colourcode) {
      case 1:
	output.append("<font color=\"#d6d3d6\">");
	break;
      case 2:
	output.append("<font color=\"#0000ce\">");
	break;
      case 3:
	output.append("<font color=\"#00ae10\">");
	break;
      case 4:
	output.append("<font color=\"#de2400\">");
	break;
      case 5:
	output.append("<font color=\"#dedb00\">");
	break;
      case 6:
	output.append("<font color=\"#de24ad\">");
	break;
      case 7:
	output.append("<font color=\"#ff9200\">");
	break;
      case 8:
	output.append("<font color=\"#ffdb52\">");
	break;
      case 9:
	output.append("<font color=\"#5adb52\">");
	break;
      case 10:
	output.append("<font color=\"#31cfce\">");
	break;
      case 11:
	output.append("<font color=\"#4adbff\">");
	break;
      case 12:
	output.append("<font color=\"#0000ff\">");
	break;
      case 13:
	output.append("<font color=\"#9449ff\">");
	break;
      case 14:
	output.append("<font color=\"#848284\">");
	break;
      case 15:
	output.append("<font color=\"#9c9a9c\">");
	break;
      }
      colour = true;
      //FIXME parse bg colours?
      break;
    default:
      output.append(c);
    }
  }

  if(bold)
    output.append("</b>");

  return output;
}

QString ircview::stripCodes( QString line )
{

}
