/*
 * connectbox.cpp
 *
 * Copyright 2001 Matthew Stephenson
 * 
 * This file is part of Zic.
 * 
 * Zic is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Zic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Zic; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Look, feel, and userlist icons shamelessly stolen from X-Chat 
 * (www.xchat.org)
 * 
 */


#include "presetbox.h"


presetbox::presetbox(QList<preset> *presets, int index, bool Adding = false)
  : QDialog(0, "foo", TRUE)
{
  this->presets = presets;
  this->Adding = Adding;
  this->index = index;

  if(Adding)
    setCaption(tr("Add Preset"));
  else
    setCaption(tr("Edit Preset"));

  vbox = new QVBox(this);
  
  grid = new QGrid(2, vbox);
  
  new QLabel("Preset name: ", grid);
  namebox = new QLineEdit(grid);
  if(!Adding)
    namebox->setText(presets->at(index)->name);

  new QLabel("Nickname: ", grid);
  nickbox = new QLineEdit(grid);
  if(!Adding)
    nickbox->setText(presets->at(index)->nick);
  
  new QLabel("Alt Nick: ", grid);
  altnickbox = new QLineEdit(grid);
  altnickbox->setEnabled(false); //FIXME: not implemented yet
  altnickbox->setText("Not implemented yet, sorry");

  new QLabel("Server: ", grid);
  serverbox = new QLineEdit(grid);
  if(!Adding)
    serverbox->setText(presets->at(index)->server);

  new QLabel("Port: ", grid);
  portbox = new QLineEdit(grid);
  if(Adding)
    portbox->setText("6667");
  else
    portbox->setText(QString::number(presets->at(index)->port));
  
  new QLabel("Password: ", grid);
  passwordbox = new QLineEdit(grid);
  passwordbox->setEnabled(false); //FIXME: not implemented yet
  passwordbox->setText("Not implemented yet, sorry");
  
  new QLabel("Channels: ", grid);
  channelsbox = new QLineEdit(grid);
  if(!Adding)
    channelsbox->setText(presets->at(index)->channels.join(","));

  new QLabel("Perform these commands on connect:", vbox);
  performbox = new QMultiLineEdit(vbox);
  performbox->setEnabled(false); //FIXME: not implemented yet
  performbox->setText("Not implemented yet, sorry");

  buttonbox = new QHBox(vbox);

  okbutton = new QPushButton("OK", buttonbox);
  okbutton->setDefault(true);
  connect(okbutton, SIGNAL(clicked()), this, SLOT(accept()));

  cancelbutton = new QPushButton("Cancel", buttonbox);
  connect(cancelbutton, SIGNAL(clicked()), this, SLOT(reject()));
  
  vbox->setFixedSize(200,250);

  this->setFixedSize(vbox->size());
}

presetbox::~presetbox()
{

}

void presetbox::accept()
{
  if(namebox->text().length() == 0) {
    QMessageBox::information(0, "Zic", "You must specify a preset name");
    return;
  }

  if(nickbox->text().length() == 0) {
    QMessageBox::information(0, "Zic", "You must specify a nickname");
    return;
  }
  else if(nickbox->text().contains(' ')) {
    QMessageBox::information(0, "Zic", 
			     "You cannot have a space in your nickname");
    return;
  }
  
  if(serverbox->text().length() == 0) {
    QMessageBox::information(0, "Zic", "You must specify an IRC server");
    return;
  }
  else if(serverbox->text().contains(' ')) {
    QMessageBox::information(0, "Zic", 
			     "You cannot have a space in the server name");
    return;
  }


  // FIXME Check we haven't got a preset with that name already
  
  if(Adding)
    pst = new preset();
  else
    pst = presets->at(index);
  
  // FIXME do other stuff than server and nick
  pst->name = namebox->text();
  pst->nick = nickbox->text();
  pst->server = serverbox->text();
  pst->port = portbox->text().toUInt();
  pst->channels = QStringList::split
    (QRegExp("[, ]"), channelsbox->text());

  if(Adding) {
    if(index == -1)
      presets->append(pst);
    else
      presets->insert(index,pst);
  }

  done(QDialog::Accepted);

}
